"""
PowerBin: Fast Adaptive Data Binning with Centroidal Power Diagrams
Copyright (C) 2025 Michele Cappellari
"""

# Makes the main PowerBin class available directly from the package, e.g.,
# from powerbin import PowerBin
from .powerbin import PowerBin

__version__ = "1.1.10"
__author__ = "Michele Cappellari"

__all__ = ['PowerBin']